@@ -1,13 +1,13 @@
-# Copyrights 2024 by [Mark Overmeer].
+# Copyrights 2024-2025 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.03.
 # SPDX-FileCopyrightText: 2024 Mark Overmeer <mark@overmeer.net>
 # SPDX-License-Identifier: Artistic-2.0
 
-package Couch::DB::Cluster;
-use vars '$VERSION';
-$VERSION = '0.006';
+package Couch::DB::Cluster;{
+our $VERSION = '0.200';
+}
 
 
 use Couch::DB::Util  qw/flat/;;
@@ -96,7 +96,7 @@ sub resharding(%)
 
 
 sub __jobValues($$)
-{	my ($couch, $job) = @_;
+{	my ($self, $couch, $job) = @_;
 
 	$couch->toPerl($job, isotime => qw/start_time update_time/)
 	      ->toPerl($job, node => qw/node/);
@@ -106,11 +106,11 @@ sub __jobValues($$)
 }
 
 sub __reshardJobsValues($$)
-{	my ($result, $data) = @_;
+{	my ($self, $result, $data) = @_;
 	my $couch  = $result->couch;
 
 	my $values = dclone $data;
-	__jobValues($couch, $_) for @{$values->{jobs} || []};
+	$self->__jobValues($couch, $_) for @{$values->{jobs} || []};
 	$values;
 }
 
@@ -119,13 +119,15 @@ sub reshardJobs(%)
 
 	$self->couch->call(GET => '/_reshard/jobs',
 		introduced => '2.4.0',
-		$self->couch->_resultsConfig(\%args, on_values => \&__reshardJobsValues),
+		$self->couch->_resultsConfig(\%args,
+			on_values => sub { $self->__reshardJobsValues(@_) },
+		),
 	);
 }
 
 
 sub __reshardStartValues($$)
-{	my ($result, $data) = @_;
+{	my ($self, $result, $data) = @_;
 	my $values = dclone $data;
 	$result->couch->toPerl($_, node => 'node')
 		for @$values;
@@ -139,17 +141,19 @@ sub reshardStart($%)
 	$self->couch->call(POST => '/_reshard/jobs',
 		introduced => '2.4.0',
 		send       => $create,
-		$self->couch->_resultsConfig(\%args, on_values => \&__reshardStartValues),
+		$self->couch->_resultsConfig(\%args,
+			on_values => sub { $self->__reshardStartValues(@_) },
+		),
 	);
 }
 
 
 sub __reshardJobValues($$)
-{	my ($result, $data) = @_;
+{	my ($self, $result, $data) = @_;
 	my $couch  = $result->couch;
 
 	my $values = dclone $data;
-	__jobValues($couch, $values);
+	$self->__jobValues($couch, $values);
 	$values;
 }
 
@@ -158,7 +162,8 @@ sub reshardJob($%)
 
 	$self->couch->call(GET => "/_reshard/jobs/$jobid",
 		introduced => '2.4.0',
-		$self->couch->_resultsConfig(\%args, on_values => \&__reshardJobValues),
+		$self->couch->_resultsConfig(\%args,
+			on_values => sub { $self->__reshardJobValues(@_) }),
 	);
 }
 
@@ -200,7 +205,7 @@ sub reshardJobChange($%)
 
 
 sub __dbshards($$)
-{	my ($result, $data) = @_;
+{	my ($self, $result, $data) = @_;
 	my $couch  = $result->couch;
 
 	my %values = %$data;
@@ -214,7 +219,9 @@ sub shardsForDB($%)
 
 	$self->couch->call(GET => $db->_pathToDB('_shards'),
 		introduced => '2.0.0',
-		$self->couch->_resultsConfig(\%args, on_values => \&__dbshards),
+		$self->couch->_resultsConfig(\%args,
+			on_values => sub { $self->__dbshards(@_) },
+		),
 	);
 }
 
@@ -232,7 +239,9 @@ sub shardsForDoc($%)
 
 	$self->couch->call(GET => $db->_pathToDB('_shards/'.$doc->id),
 		introduced => '2.0.0',
-		$self->couch->_resultsConfig(\%args, on_values => \&__docshards),
+		$self->couch->_resultsConfig(\%args,
+			on_values => sub { $self->__docshards(@_) },
+		),
 	);
 }
 
