@@ -1,17 +1,18 @@
-# Copyrights 2024 by [Mark Overmeer].
+# Copyrights 2024-2025 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.03.
 # SPDX-FileCopyrightText: 2024 Mark Overmeer <mark@overmeer.net>
 # SPDX-License-Identifier: Artistic-2.0
 
-package Couch::DB::Result;
-use vars '$VERSION';
-$VERSION = '0.006';
+package Couch::DB::Result;{
+our $VERSION = '0.200';
+}
 
 
 use Couch::DB::Util     qw(flat pile);
 use Couch::DB::Document ();
+use Couch::DB::Row      ();
 
 use Log::Report   'couch-db';
 use HTTP::Status  qw(is_success status_constant_name HTTP_OK HTTP_CONTINUE HTTP_MULTIPLE_CHOICES);
@@ -25,9 +26,13 @@ my %default_code_texts = (  # do not construct them all the time again
 	&HTTP_MULTIPLE_CHOICES	=> 'The Result object does not know what to do, yet.',
 );
 
+my $seqnr = 0;
+
 
 use overload
-	bool => sub { $_[0]->code < 400 };
+	bool     => sub { $_[0]->code < 400 },
+	'""'     => 'short',
+	fallback => 1;
 
 
 sub new(@) { my ($class, %args) = @_; (bless {}, $class)->init(\%args) }
@@ -36,14 +41,14 @@ sub init($)
 {	my ($self, $args) = @_;
 
 	$self->{CDR_couch}     = delete $args->{couch} or panic;
-	weaken $self->{CDR_couch};
-
 	$self->{CDR_on_final}  = pile delete $args->{on_final};
 	$self->{CDR_on_error}  = pile delete $args->{on_error};
 	$self->{CDR_on_chain}  = pile delete $args->{on_chain};
 	$self->{CDR_on_values} = pile delete $args->{on_values};
+	$self->{CDR_on_row}    = pile delete $args->{on_row};
 	$self->{CDR_code}      = HTTP_MULTIPLE_CHOICES;
 	$self->{CDR_page}      = delete $args->{paging};
+	$self->{CDR_seqnr}     = ++$seqnr;
 
 	$self;
 }
@@ -71,13 +76,27 @@ sub message()
 }
 
 
-sub status($$)
+sub setStatus($$)
 {	my ($self, $code, $msg) = @_;
 	$self->{CDR_code} = $code;
 	$self->{CDR_msg}  = $msg;
 	$self;
 }
 
+
+sub seqnr() { $_[0]->{CDR_seqnr} }
+
+
+sub short()
+{	my $self = shift;
+	my $client = $self->client;
+	my $req    = $self->request;
+
+	$client && $req
+	  ? (sprintf "RESULT %07d.%08d %-6s %s\n", $client->seqnr, $self->seqnr, $req->method, $req->url =~ s/\?.*/?.../r)
+	  : (sprintf "RESULT prepare.%08d\n", $self->seqnr);
+}
+
 #-------------
 
 sub client()    { $_[0]->{CDR_client} }
@@ -94,7 +113,7 @@ sub answer(%)
  	$self->isReady
 		or error __x"Document not ready: {err}", err => $self->message;
 
-	$self->{CDR_answer} = $self->couch->_extractAnswer($self->response),
+	$self->{CDR_answer} = $self->couch->_extractAnswer($self->response);
 }
 
 
@@ -109,6 +128,71 @@ sub values(@)
 
 #-------------
 
+sub rows(;$) { @{$_[0]->rowsRef($_[1])} }
+
+
+sub rowsRef(;$)
+{	my ($self, $qnr) = @_;
+
+	! $self->inPagingMode
+		or panic "Call used in paging mode, so use the page* methods.";
+
+	$self->_rowsRef($qnr // 0);
+}
+
+sub _rowsRef($)
+{	my ($self, $qnr) = @_;
+	my $rows = $self->{CDR_rows}[$qnr] ||= [];
+	return $rows if $self->{CDR_rows_complete}[$qnr];
+
+	for(my $rownr = 1; $self->row($rownr, $qnr); $rownr++) { }
+	$self->{CDR_rows_complete}[$qnr] = 1;
+	$rows;
+}
+
+
+sub row($;$)
+{	my ($self, $rownr, $qnr) = @_;
+	my $rows  = $self->{CDR_rows}[$qnr //= 0] ||= [];
+	my $index = $rownr -1;
+	return $rows->[$index] if exists $rows->[$index];
+
+	my %data = map $_->($self, $rownr-1, column => $qnr), reverse @{$self->{CDR_on_row}};
+	keys %data or return ();
+
+	my $doc;
+	my $dp = delete $data{docparams} || {};
+	if(my $dd = delete $data{docdata})
+	{	$doc  = Couch::DB::Document->fromResult($self, $dd, %$dp);
+	}
+	elsif($dd = delete $data{ddocdata})
+	{	$doc  = Couch::DB::Design->fromResult($self, $dd, %$dp);
+	}
+
+	my $row = Couch::DB::Row->new(%data, result => $self, rownr => $rownr, doc => $doc);
+	$doc->row($row) if $doc;
+
+	$rows->[$index] = $row;    # Remember partial result for rows()
+}
+
+
+sub numberOfRows(;$) { scalar @{$_[0]->rowsRef($_[1])} }
+
+
+sub docs(;$) { map $_->doc, $_[0]->rows($_[1]) }
+
+
+sub docsRef(;$) { [ map $_->doc, $_[0]->rows($_[1]) ] }
+
+
+sub doc($;$)
+{	my ($self, $rownr, $qnr) = @_;
+	my $r = $self->row($rownr, $qnr);
+	defined $r ? $r->doc : undef;
+}
+
+#-------------
+
 sub pagingState(%)
 {	my ($self, %args) = @_;
 	my $next = $self->nextPageSettings;
@@ -124,7 +208,13 @@ sub pagingState(%)
 	$next;
 }
 
-# The next is used r/w when _succeed is a result object, and when results
+
+sub supportsPaging() { defined $_[0]->{CDR_page} }
+
+
+sub inPagingMode() { my $r = $_[0]->{CDR_page}; $r && $r->{page_mode} }
+
+# The next is used r/w when succeed is a result object, and when results
 # have arrived.
 
 sub _thisPage() { $_[0]->{CDR_page} or panic "Call does not support paging." }
@@ -134,34 +224,54 @@ sub nextPageSettings()
 {	my $self = shift;
 	my %next = %{$self->_thisPage};
 	delete $next{harvested};
-	$next{start} += (delete $next{skip}) + @{$self->page};
-#use Data::Dumper;
-#warn "NEXT PAGE=", Dumper \%next;
+	$next{start} += (delete $next{skip}) + @{$self->_rowsRef(0)};
+	$next{pagenr}++;
 	\%next;
 }
 
 
-sub page() { $_[0]->_thisPage->{harvested} }
+sub page()
+{	my $self = shift;
 
-sub _pageAdd($@)
-{	my $this     = shift->_thisPage;
-	my $bookmark = shift;
-	my $page     = $this->{harvested};
-	if(@_)
-	{	push @$page, @_;
-		$this->{bookmarks}{$this->{start} + $this->{skip} + @$page} = $bookmark
-			if defined $bookmark;
-	}
-	else
-	{	$this->{end_reached} = 1;
+	$self->inPagingMode
+		or panic "Call not in paging mode, use the row* and doc* alternative methods.";
+
+	$self->_thisPage->{harvested};
+}
+
+sub _pageAdd($$)
+{	my ($self, $bookmark, $found) = @_;
+	my $this = $self->_thisPage;
+	my $page = $this->{harvested};
+	push @$page, @$found;
+
+	if(defined $bookmark)
+	{	my $recv = $this->{start} + $this->{skip} + @$page;
+		$this->{bookmarks}{$recv} = $bookmark;
 	}
+
+	$this->{end_reached} = ! @$found || $this->{stop}->($self);
 	$page;
 }
 
 
+sub pageRows() { @{$_[0]->page} }
+
+
+sub pageNumber() { $_[0]->_thisPage->{pagenr} }
+
+
+sub pageDocs() { map $_->doc, @{$_[0]->page} }
+
+
+sub pageDoc($) { my $r = $_[0]->page->[$_[1]-1]; defined $r ? $r->doc : undef }
+
+
 sub pageIsPartial()
 {	my $this = shift->_thisPage;
-	! $this->{end_reached} && ($this->{all} || @{$this->{harvested}} < $this->{page_size});
+	     $this->{page_mode}
+	  && ! $this->{end_reached}
+	  && ($this->{all} || @{$this->{harvested}} < $this->{page_size});
 }
 
 
@@ -179,14 +289,17 @@ sub setFinalResult($%)
 	$self->{CDR_ready}    = 1;
 	$self->{CDR_request}  = delete $data->{request};
 	$self->{CDR_response} = delete $data->{response};
-	$self->status($code, delete $data->{message});
+	$self->setStatus($code, delete $data->{message});
 
 	delete $self->{CDR_answer};  # remove cached while paging
 	delete $self->{CDR_values};
+	delete $self->{CDR_rows};
 
+#warn "CODE=$code, $self";
 	# "on_error" handler
 	unless(is_success $code)
 	{	$_->($self) for @{$self->{CDR_on_error}};
+		return undef;
 	}
 
 	# "on_final" handler
@@ -211,7 +324,7 @@ sub setResultDelayed($%)
 {	my ($self, $plan, %args) = @_;
 
 	$self->{CDR_delayed}  = $plan;
-	$self->status(HTTP_CONTINUE);
+	$self->setStatus(HTTP_CONTINUE);
 	$self;
 }
 
